﻿// -*- C++ -*-
/*!
 * @file  moveCommander2.cpp
 * @brief ModuleDescription
 * @date $Date$
 *
 * $Id$
 */

#include "moveCommander2.h"

// Module specification
// <rtc-template block="module_spec">
static const char* movecommander2_spec[] =
  {
    "implementation_id", "moveCommander2",
    "type_name",         "moveCommander2",
    "description",       "ModuleDescription",
    "version",           "1.0.0",
    "vendor",            "VenderName",
    "category",          "Category",
    "activity_type",     "PERIODIC",
    "kind",              "DataFlowComponent",
    "max_instance",      "1",
    "language",          "C++",
    "lang_type",         "compile",
    // Configuration variables
    "conf.default.speed_foeward", "0.1",
    "conf.default.speed_round", "0.2",
    "conf.default.threshold_forward", "300",
    "conf.default.threshold_side", "500",
    "conf.default.error_rate", "0.001",

    // Widget
    "conf.__widget__.speed_foeward", "text",
    "conf.__widget__.speed_round", "text",
    "conf.__widget__.threshold_forward", "text",
    "conf.__widget__.threshold_side", "text",
    "conf.__widget__.error_rate", "text",
    // Constraints

    "conf.__type__.speed_foeward", "double",
    "conf.__type__.speed_round", "double",
    "conf.__type__.threshold_forward", "int",
    "conf.__type__.threshold_side", "int",
    "conf.__type__.error_rate", "double",

    ""
  };
// </rtc-template>

/*!
 * @brief constructor
 * @param manager Maneger Object
 */
moveCommander2::moveCommander2(RTC::Manager* manager)
    // <rtc-template block="initializer">
  : RTC::DataFlowComponentBase(manager),
    m_inURGIn("inURG", m_inURG),
    m_inConsoleIn("inConsole", m_inConsole),
    m_outToKobukiOut("outToKobuki", m_outToKobuki),
    m_outToViewOut("outToView", m_outToView),
    m_outToPlotOut("outToPlot", m_outToPlot)

    // </rtc-template>
{
}

/*!
 * @brief destructor
 */
moveCommander2::~moveCommander2()
{
}



RTC::ReturnCode_t moveCommander2::onInitialize()
{
  // Registration: InPort/OutPort/Service
  // <rtc-template block="registration">
  // Set InPort buffers
  addInPort("inURG", m_inURGIn);
  addInPort("inConsole", m_inConsoleIn);
  
  // Set OutPort buffer
  addOutPort("outToKobuki", m_outToKobukiOut);
  addOutPort("outToView", m_outToViewOut);
  addOutPort("outToPlot", m_outToPlotOut);
  
  // Set service provider to Ports
  
  // Set service consumers to Ports
  
  // Set CORBA Service Ports
  
  // </rtc-template>

  // <rtc-template block="bind_config">
  // Bind variables and configuration variable
  bindParameter("speed_foeward", m_speed_forward, "0.1");
  bindParameter("speed_round", m_speed_round, "0.2");
  bindParameter("threshold_forward", m_threshold_forward, "450"); //side-50mm
  bindParameter("threshold_side", m_threshold_side, "500");
  bindParameter("error_rate", m_error_rate, "0.001");
  // </rtc-template>
  
  return RTC::RTC_OK;
}

/*
RTC::ReturnCode_t moveCommander2::onFinalize()
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t moveCommander2::onStartup(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t moveCommander2::onShutdown(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/


RTC::ReturnCode_t moveCommander2::onActivated(RTC::UniqueId ec_id)
{
	//変数初期化
	counter_f = counter_s = counter_x1 = counter_x2 = 0;
	dist_f = dist_s = dist_e = 0;
	dist_x1 = dist_x2 = 0;
	fx = fy = 0;
	sx = sy = 0;
	x1 = y1 = 0;
	x2 = y2 = 0;
	state = 4; //停止
	consoleState = 0;
	counter = 0;

  return RTC::RTC_OK;
}


RTC::ReturnCode_t moveCommander2::onDeactivated(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}


RTC::ReturnCode_t moveCommander2::onExecute(RTC::UniqueId ec_id)
{
	//コンソール入力
	if (m_inConsoleIn.isNew()) {

		m_inConsoleIn.read();

		consoleState = m_inConsole.data;

		if (consoleState == 1) {
			state = 1;
		}
		else if (consoleState == 2) {
			state = 2;
		}
		else if (consoleState == 3) {
			state = 3;
		}
		else if (consoleState == 4) {
			state = 4;
		}

		cout << "consoleIn " << consoleState << endl;
	
	}


	//レンジデータ取得
	if (m_inURGIn.isNew()) {
		//新しいデータが来ていたら

		m_inURGIn.read(); //データ読み込み
		m_size = m_inURG.ranges.length(); //配列長取得
		m_outToView.ranges.length(m_size); //m_outToViewの配列を確保

		//全てのデータをコピー
		m_outToView.config = m_inURG.config;
		m_outToView.geometry = m_inURG.geometry;
		m_outToView.tm = m_inURG.tm;

		//変数初期化
		counter_f = counter_s = counter_x1 = counter_x2 = 0;
		dist_f = dist_s = dist_e = 0;
		dist_x1 = dist_x2 = 0;
		fx = fy = 0;
		sx = sy = 0;
		x1 = y1 = 0;
		x2 = y2 = 0;

		//レンジデータ読み取り
		for (int i = 0; i < m_size; i++) {
			m_inURG.ranges[i] *= 1000; //単位変換 [m]→[mm]

			//壁側
			//x1
			if (i == 430 || i == 440 || i == 450 || i == 460 || i == 470) {
				x1 += m_inURG.ranges[i] * cos(((i - 431)*0.532)*(3.14 / 180.0));
				counter_x1++;
			}
			//壁との距離
			else if (i == 320 || i == 330 || i == 340 || i == 350 || i == 360) {
				sx += m_inURG.ranges[i] * cos(((i - 341)*0.352)* (3.14 / 180.0));
				counter_s++;
			}
			//x2
			else if (i == 210 || i == 220 || i == 230 || i == 240 || i == 250) {
				x2 += m_inURG.ranges[i] * cos(((i - 213)*0.532)*(3.14 / 180.0));
				counter_x2++;
			}
			//前方
			else if (582 < i && i < 610) {
				//fx += m_inURG.ranges[i] * cos(((i - 341)*0.352)* (3.14 / 180.0));
				fy += m_inURG.ranges[i] * sin(((i - 341)*0.352)* (3.14 / 180.0));
				counter_f++;
			}
			//捨てるデータ
			else {
				m_inURG.ranges[i] = 0.0;
			}

			//出力データに代入
			m_outToView.ranges[i] = m_inURG.ranges[i];

		}

		//平均距離算出
		if (counter_f != 0) dist_f = fy / counter_f;
		if (counter_s != 0) dist_s = sx / counter_s;
		if (counter_x1 != 0) dist_x1 = x1 / counter_x1;
		if (counter_x2 != 0) dist_x2 = x2 / counter_x2;

		//壁に対するずれ
		dist_e = dist_x2 - dist_x1;

		//View画面へ出力
		m_outToViewOut.write();
		
		//状態遷移
		if (state == 1 && (200 < dist_f && dist_f < m_threshold_forward)) {
			state = 2; //壁に遭遇
			counter++;

			if (counter >= 2) { //2回壁を認識したら停止
				state = 4;
				m_outToPlot.data = true;
				m_outToPlotOut.write();
			}
		}

		//ロボット移動
		//前進
		if (state == 1) {
			
			m_outToKobuki.data.va = 0.0;
			m_outToKobuki.data.vx = m_speed_forward;
			m_outToKobuki.data.vy = 0.0;

			//壁に並走するように制御する部分
			//本番では使用しなかった
			/*
			if (((m_threshold_side - 10) < dist_s) && (dist_s < (m_threshold_side + 10))) {
				//目標範囲に入っている
				m_outToKobuki.data.va = dist_e * m_error_rate;
				m_outToKobuki.data.vx = m_speed_forward;
				m_outToKobuki.data.vy = 0.0;
				//cout << "j " << dist_f << endl;
			}
			else {
				//走行位置が離れてる
				if (m_threshold_side < dist_s) {
					m_outToKobuki.data.va = -300 * m_error_rate;
					m_outToKobuki.data.vx = m_speed_forward;
					m_outToKobuki.data.vy = 0.0;
					//cout << "l " << dist_f << endl;
				}
				//走行位置が近い
				else {
					m_outToKobuki.data.va = 300 * m_error_rate;
					m_outToKobuki.data.vx = m_speed_forward;
					m_outToKobuki.data.vy = 0.0;
					//cout << "r " << dist_f << endl;
				}
			}
			*/

		}

		//壁に遭遇
		else if (state == 2) {
			state = 3;
		}

		//回転
		else if (state == 3) {
			//cout << "t " << dist_f << " " << dist_e << endl;
			m_outToKobuki.data.va = m_speed_round;
			m_outToKobuki.data.vx = 0.0;
			m_outToKobuki.data.vy = 0.0;

			if ((abs(dist_e) < 30) && (dist_f > 1000)) {
				state = 1;
				//cout << "c" << endl;
			}
		}
		//停止（終了）
		else if (state == 4) {
			m_outToKobuki.data.va = 0.0;
			m_outToKobuki.data.vx = 0.0;
			m_outToKobuki.data.vy = 0.0;
		}

		m_outToKobukiOut.write();

	}

  return RTC::RTC_OK;
}

/*
RTC::ReturnCode_t moveCommander2::onAborting(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t moveCommander2::onError(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t moveCommander2::onReset(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t moveCommander2::onStateUpdate(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t moveCommander2::onRateChanged(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/



extern "C"
{
 
  void moveCommander2Init(RTC::Manager* manager)
  {
    coil::Properties profile(movecommander2_spec);
    manager->registerFactory(profile,
                             RTC::Create<moveCommander2>,
                             RTC::Delete<moveCommander2>);
  }
  
};